import json

from flask import Blueprint, Response
from inventory_provider.routes import common

routes = Blueprint("inventory-opsdb-query-routes", __name__)

interfaces_key = "interface_services"
equipment_locations_key = "equipment_locations"
service_child_to_parents_key = "child_to_parent_circuit_relations"
service_parent_to_children_key = "parent_to_children_circuit_relations"
interface_status_key = "interface_statuses"


def _decode_utf8_dict(d):
    return {k.decode('utf8'): json.loads(v) for k, v in d.items()}


@routes.route("/interfaces")
def get_all_interface_details():
    r = common.get_redis()
    result = _decode_utf8_dict(
            r.hgetall(interfaces_key))

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/interfaces/<equipment_name>")
def get_interface_details_for_equipment(equipment_name):
    r = common.get_redis()
    result = {}
    for t in r.hscan_iter(interfaces_key, "{}::*".format(equipment_name)):
        result[t[0].decode("utf8")] = json.loads(t[1])

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/interfaces/<equipment_name>/<path:interface>")
def get_interface_details(equipment_name, interface):
    r = common.get_redis()
    return Response(
        r.hget(
            interfaces_key,
            "{}::{}".format(equipment_name, interface)),
        mimetype="application/json")


@routes.route("/equipment-location")
def get_all_equipment_locations():
    r = common.get_redis()
    result = list(
        _decode_utf8_dict(
            r.hgetall(equipment_locations_key)).values())

    return Response(
        json.dumps(result),
        mimetype="application/json")


@routes.route("/equipment-location/<path:equipment_name>")
def get_equipment_location(equipment_name):
    r = common.get_redis()
    return Response(
        r.hget(equipment_locations_key, equipment_name),
        mimetype="application/json")


@routes.route("/circuit-hierarchy/children/<parent_id>")
def get_children(parent_id):
    r = common.get_redis()
    return Response(
        r.hget(
            service_parent_to_children_key,
            parent_id),
        mimetype="application/json")


@routes.route("/circuit-hierarchy/parents/<child_id>")
def get_parents(child_id):
    r = common.get_redis()
    return Response(
        r.hget(
            service_child_to_parents_key,
            child_id),
        mimetype="application/json")
